import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {ScriptAccountCondition} from "../../../game-model/gamesystems/conditions/ScriptAccountCondition";
import {MatTableDataSource} from "@angular/material/table";
import {ScriptAccount} from "../../../game-model/scriptAccounts/ScriptAccount";

@Component({
  selector: 'app-scriptaccount-condition-editor',
  templateUrl: './scriptaccount-condition-editor.component.html',
  styleUrl: './scriptaccount-condition-editor.component.scss'
})
export class ScriptaccountConditionEditorComponent implements OnInit{

  @Input() conditions: ScriptAccountCondition[] = []
  @Input() scriptAccounts: ScriptAccount[] = []
  @Output() onCreateCondition: EventEmitter<ScriptAccountCondition> = new EventEmitter<ScriptAccountCondition>();
  @Output() onDeleteCondition: EventEmitter<ScriptAccountCondition> = new EventEmitter<ScriptAccountCondition>();

  dataSource: MatTableDataSource<ScriptAccountCondition> = new MatTableDataSource();
  displayedColumns = ['scriptAccount', 'minValue', 'maxValue', 'edit', 'delete']

  editedCondition: ScriptAccountCondition | undefined
  addedCondition: ScriptAccountCondition | undefined

  ngOnInit() {
    this.dataSource.data = this.conditions.concat();
  }

  createCondition() {
    this.addedCondition = ScriptAccountCondition.constructScriptAccountCondition(new ScriptAccount("", ""), 0,0);
    this.editedCondition = this.addedCondition;
    this.dataSource.data = this.dataSource.data.concat(this.addedCondition!);
  }

  finishEditing() {
    if(this.addedCondition != undefined) {
      const createdCondition = ScriptAccountCondition.constructScriptAccountCondition(this.addedCondition.scriptAccount, this.addedCondition.minValue, this.addedCondition.maxValue);
      if(createdCondition != undefined) {
        console.log(createdCondition)
        this.onCreateCondition.emit(createdCondition);
        console.log(this.conditions)
        this.dataSource.data = this.conditions;
      }
      this.addedCondition = undefined;
    }
    this.editedCondition = undefined;
  }

  edit(condition: ScriptAccountCondition) {
    this.editedCondition = condition;
  }

  deleteCondition(condition: ScriptAccountCondition) {
    if(this.addedCondition === condition) {
      this.addedCondition = undefined;
      this.dataSource.data = this.conditions.concat();
    } else {
      this.onDeleteCondition.emit(condition);
      this.dataSource.data = this.conditions.concat();
    }
  }
}
