import {Component, EventEmitter, Input, OnInit, Output} from '@angular/core';
import {Gamesystem} from "../../game-model/gamesystems/Gamesystem";
import {State} from "../../game-model/gamesystems/states/State";
import {Transition} from "../../game-model/gamesystems/transitions/Transition";
import {ProductGamesystem} from "../../game-model/gamesystems/ProductGamesystem";
import {FlatTreeControl} from "@angular/cdk/tree";
import {
  MatTree,
  MatTreeFlatDataSource,
  MatTreeFlattener,
  MatTreeNode, MatTreeNodeDef,
  MatTreeNodePadding, MatTreeNodeToggle
} from "@angular/material/tree";
import {MatIcon} from "@angular/material/icon";
import {MatIconButton} from "@angular/material/button";
import {SimpleGamesystem} from "../../game-model/gamesystems/SimpleGamesystem";
import {GameModel} from "../../game-model/GameModel";
import {ElectronService} from "../../core/services";


interface FlatNode {
  expandable: boolean,
  name: string,
  level: number
}
@Component({
  selector: 'app-gamescript-overview',
  templateUrl: './gamescript-overview.component.html',
  styleUrl: './gamescript-overview.component.scss'
})
export class GamescriptOverviewComponent implements OnInit {

  @Input('gameModel') gameModel: GameModel | undefined
  @Output('openGamesystemEditor') openGamesystemEmitter : EventEmitter<Gamesystem<State<any>, Transition<any>>> = new EventEmitter<Gamesystem<State<any>, Transition<any>>>();


  ngOnInit() {
    this.dataSource.data = this.gameModel!.gamesystems;
  }

  private _transformer = (node: Gamesystem<State<any>, Transition<any>>, level: number) => {
    return {
      expandable: this.isProductGamesystem(node) && !!(node as ProductGamesystem).innerGamesystems && (node as ProductGamesystem).innerGamesystems.length > 0,
      name: node.componentName,
      level: level
    }
  }

  treeControl = new FlatTreeControl<FlatNode>(
    node => node.level,
    node => node.expandable
  )

  treeFlattener = new MatTreeFlattener(
    this._transformer,
    node => node.level,
    node => node.expandable,
    node => this.isSimpleGamesystem(node)? []: (node as ProductGamesystem).innerGamesystems
  );

  dataSource = new MatTreeFlatDataSource(this.treeControl, this.treeFlattener);
  selectedGamesystem: FlatNode | undefined;

  constructor(private electronService: ElectronService) {

  }

  hasChild = (_: number, node: FlatNode) => node.expandable;


  isSimpleGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    return gamesystem instanceof SimpleGamesystem;
  }

  isProductGamesystem(gamesystem: Gamesystem<State<any>, Transition<any>>) {
    return gamesystem instanceof ProductGamesystem;
  }

  onSelectGamesystem(node: FlatNode) {
    this.selectedGamesystem = node;
  }

  onContextMenu(event: MouseEvent) {
    this.electronService.ipcRenderer.send('context-menu', {x: event.x, y: event.y});
  }

  openGamesystemEditor(node: FlatNode) {
    const gamesystem: Gamesystem<State<any>, Transition<any>>| undefined= this.gameModel!.findGamesystem(node.name);
    if(gamesystem != undefined) {
      gamesystem.unsaved = false;
      this.openGamesystemEmitter.emit(gamesystem);
    }
  }

  get selectedGamesystemName() {
    if(this.selectedGamesystem == undefined) {
      return undefined
    } else {
      return this.selectedGamesystem!.name
    }
  }

  refresh() {
    this.dataSource.data = this.gameModel!.gamesystems;
  }
}
